﻿using System;
using System.Collections.ObjectModel;

using Prism.Commands;
using Prism.Mvvm;
using Prism.Regions;

using TestLibrary.Business;

namespace TestLibrary.ViewModels
{
    /// <summary>
    /// 사람 리스트 뷰 모델
    /// </summary>
    public class PersonListViewModel : BindableBase
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 영역 관리자 인터페이스
        /// </summary>
        private IRegionManager regionManager;

        /// <summary>
        /// 사람 컬렉션
        /// </summary>
        private ObservableCollection<Person> personCollection;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 사람 컬렉션 - PersonCollection

        /// <summary>
        /// 사람 컬렉션
        /// </summary>
        public ObservableCollection<Person> PersonCollection
        {
            get
            {
                return this.personCollection;
            }
            set
            {
                SetProperty(ref this.personCollection, value);
            }
        }

        #endregion
        #region 사람 선택 명령 - SelectPersonCommand

        /// <summary>
        /// 사람 선택 명령
        /// </summary>
        public DelegateCommand<Person> SelectPersonCommand { get; private set; }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - PersonListViewModel(regionManager)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="regionManager">영역 관리자 인터페이스</param>
        public PersonListViewModel(IRegionManager regionManager)
        {
            this.regionManager = regionManager;

            SelectPersonCommand = new DelegateCommand<Person>(SelectPerson);

            ObservableCollection<Person> personCollection = new ObservableCollection<Person>();

            for(int i = 0; i < 10; i++)
            {
                personCollection.Add
                (
                    new Person()
                    {
                        FirstName = string.Format("First {0}", i),
                        LastName  = string.Format("Last {0}" , i),
                        Age       = i
                    }
                );
            }

            PersonCollection = personCollection;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 사람 선택하기 - SelectPerson(person)

        /// <summary>
        /// 사람 선택하기
        /// </summary>
        /// <param name="person">사람</param>
        private void SelectPerson(Person person)
        {
            NavigationParameters parameters = new NavigationParameters();

            parameters.Add("person", person);

            if(person != null)
            {
                this.regionManager.RequestNavigate("PersonRegion", "PersonView", parameters);
            }
        }

        #endregion
    }
}